#ifndef __PLX_PCI_FUNCTIONS_H
#define __PLX_PCI_FUNCTIONS_H

/*******************************************************************************
 * Copyright (c) PLX Technology, Inc.
 *
 * PLX Technology Inc. licenses this source file under the GNU Lesser General Public
 * License (LGPL) version 2.  This source file may be modified or redistributed
 * under the terms of the LGPL and without express permission from PLX Technology.
 *
 * PLX Technology, Inc. provides this software AS IS, WITHOUT ANY WARRANTY,
 * EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION, ANY WARRANTY OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  PLX makes no guarantee
 * or representations regarding the use of, or the results of the use of,
 * the software and documentation in terms of correctness, accuracy,
 * reliability, currentness, or otherwise; and you rely on the software,
 * documentation and results solely at your own risk.
 *
 * IN NO EVENT SHALL PLX BE LIABLE FOR ANY LOSS OF USE, LOSS OF BUSINESS,
 * LOSS OF PROFITS, INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES
 * OF ANY KIND.
 *
 ******************************************************************************/

/******************************************************************************
 *
 * File Name:
 *
 *      PciFunc.h
 *
 * Description:
 *
 *      The header file for PCI support functions
 *
 * Revision History:
 *
 *      12-01-07 : PLX SDK v5.20
 *
 ******************************************************************************/


#include <linux/pci.h>
#include "DrvDefs.h"




/**********************************************
 *               Definitions
 *********************************************/
#if !defined(PCI_ANY_ID)
    #define PCI_ANY_ID                      (~0)
#endif

// Macros for PCI register access
#define PLX_PCI_REG_READ(pdx, offset, pValue) \
    pci_read_config_dword(   \
        (pdx)->pPciDevice,   \
        (U16)(offset),       \
        (pValue)             \
        )

#define PLX_PCI_REG_WRITE(pdx, offset, value) \
    pci_write_config_dword(  \
        (pdx)->pPciDevice,   \
        (U16)(offset),       \
        (value)              \
        )

// Unsupported calls not needed for Linux - bus scan is allowed
#define PlxPciRegisterRead_Unsupported       PlxPciRegisterRead
#define PlxPciRegisterWrite_Unsupported      PlxPciRegisterWrite




/**********************************************
 *               Functions
 *********************************************/
PLX_STATUS
PlxPciRegisterRead(
    U8   bus,
    U8   slot,
    U8   function,
    U16  offset,
    U32 *pValue
    );

PLX_STATUS
PlxPciRegisterWrite(
    U8  bus,
    U8  slot,
    U8  function,
    U16 offset,
    U32 value
    );



#endif
