/******************************************************************************
 *
 * File Name:
 *
 *      NTSample.c
 *
 * Description:
 *
 *      This sample demonstrates basic communication across an NT port between
 *      two custom PLX applications.
 *
 * Revision History:
 *
 *      07-01-08 : PLX SDK v6.00
 *
 ******************************************************************************/


#include <ctype.h>
#include "PlxApi.h"

#if defined(_WIN32)
    #include "..\\Shared\\ConsFunc.h"
#endif

#if defined(PLX_LINUX)
    #include "ConsFunc.h"
#endif




/**********************************************
 *               Definitions
 *********************************************/
#define NT_PCI_BUFFER_SIZE                  (1000)          // Size of buffer for receiving data
#define MAX_DEVICES_TO_LIST                 50              // Max number of devices for user selection

#define NT_MSG_SYSTEM_READY                 0xFEEDFACE      // Code passed between systems to signal ready

#define PLX_8500_NT_SCRATCH_REG_0           0xB0            // Scratch regitser 0
#define PLX_8500_NT_SCRATCH_REG_1           0xB4            // Scratch register 1
#define PLX_8500_NT_SCRATCH_REG_2           0xB8            // Scratch register 2
#define PLX_8500_NT_SCRATCH_REG_3           0xBC            // Scratch register 3
#define PLX_8500_NT_SCRATCH_REG_4           0xC0            // Scratch register 4
#define PLX_8500_NT_SCRATCH_REG_5           0xC4            // Scratch register 5
#define PLX_8500_NT_SCRATCH_REG_6           0xC8            // Scratch register 6
#define PLX_8500_NT_SCRATCH_REG_7           0xCC            // Scratch register 7

#define PLX_8600_NT_SCRATCH_REG_0           0xC6C           // Scratch regitser 0
#define PLX_8600_NT_SCRATCH_REG_1           0xC70           // Scratch register 1
#define PLX_8600_NT_SCRATCH_REG_2           0xC74           // Scratch register 2
#define PLX_8600_NT_SCRATCH_REG_3           0xC78           // Scratch register 3
#define PLX_8600_NT_SCRATCH_REG_4           0xC7C           // Scratch register 4
#define PLX_8600_NT_SCRATCH_REG_5           0xC80           // Scratch register 5
#define PLX_8600_NT_SCRATCH_REG_6           0xC84           // Scratch register 6
#define PLX_8600_NT_SCRATCH_REG_7           0xC88           // Scratch register 7


////////////// Direct Address Translation ////////////
#define NT_MEMORY_BAR_2_TRAN_LOWER          0xC3C           // Memory BAR 2 lower 32-address translation reg.
#define NT_MEMORY_BAR_2_TRAN_UPPER          0xC40           // Memory BAR 2 upper 32-address translation reg.
#define NT_MEMORY_BAR_4_TRAN_LOWER          0xC44           // Memory BAR 4 lower 32-address translation reg.
#define NT_MEMORY_BAR_4_TRAN_UPPER          0xC48           // Memory BAR 4 upper 32-address translation reg.
#define NT_MEMORY_BAR_2_LIMIT_LOWER         0xC4C           // Memory BAR 2 limit lower 32-address
#define NT_MEMORY_BAR_2_LIMIT_UPPER         0xC50           // Memory BAR 2 limit upper 32-address
#define NT_MEMORY_BAR_4_LIMIT_LOWER         0xC54           // Memory BAR 4 limit lower 32-address
#define NT_MEMORY_BAR_4_LIMIT_UPPER         0xC58           // Memory BAR 4 limit upper 32-address




/**********************************************
 *              Definitions
 *********************************************/
#define PlxBarMem_32(Va, offset)               *(VU32*)((U8*)Va + offset)


// PLX Direct Address Translation Strture
typedef struct _PLX_DIRECT_ADDRESS
{
    U64 DestinationAddr;
    U64 Size;
} PLX_DIRECT_ADDRESS;




/**********************************************
 *               Functions
 *********************************************/
S8
SelectDevice_NT(
    PLX_DEVICE_KEY *pKey,
    U32            *pPlxChip,
    BOOLEAN        *pbLinkSide
    );

BOOLEAN
PlxDetermineNtPortSide(
    PLX_DEVICE_OBJECT *pDevice,
    BOOLEAN           *pbLinkSide
    );

BOOLEAN
PlxDetermineRequesterID(
    PLX_DEVICE_OBJECT *pDevice,
    U32               *VaBar2,
    U16               *pReqId
    );

BOOLEAN
PlxAddLutEntry(
    PLX_DEVICE_OBJECT *pDevice,
    U16                index,
    U16                ReqId,
    U32                PlxChip,
    BOOLEAN            bLinkSide
    );

S8
WaitForConnection(
    PLX_DEVICE_OBJECT *pDevice,
    U32                PlxChip,
    BOOLEAN            bLinkSide
    );

PLX_STATUS
PlxPci_SetupNtTranslation(
    PLX_DEVICE_OBJECT  *pDevice,
    U8                  BarIndex,
    PLX_DIRECT_ADDRESS  DirAddr,
    BOOLEAN             bLinkSide
    );

U32
PlxPci_PlxMailboxRead(
    PLX_DEVICE_OBJECT *pDevice,
    S8                 Mailbox,
    PLX_STATUS        *pStatus,
    U32                PlxChip
    );

PLX_STATUS
PlxPci_PlxMailboxWrite(
    PLX_DEVICE_OBJECT *pDevice,
    S8                 Mailbox,
    U32                value,
    U32                PlxChip
    );




/******************************************************************************
 *
 * Function   :  main
 *
 * Description:  The main entry point
 *
 *****************************************************************************/
int
main(
    void
    )
{
    char                CharRead;
    S8                  DeviceSelected;
    U8                  BarNum;
    U16                 ReqId;
    U32                 size;
    U32                 value;
    U32                 PlxChip;
    U32                 BarOffset;
    U32                 PhysAddr;
    VU32               *pSysMem;
    VOID               *BarVa;
    BOOLEAN             bLinkSide;
    PLX_STATUS          status;
    PLX_DEVICE_KEY      DeviceKey;
    PLX_PHYSICAL_MEM    PhysBuffer;
    PLX_PCI_BAR_PROP    BarProp;
    PLX_DEVICE_OBJECT   Device;
    PLX_DIRECT_ADDRESS  DirAddr;


    // Mark physical buffer as not allocated
    PhysBuffer.PhysicalAddr = 0;

    ConsoleInitialize();

    Cons_clear();

    Cons_printf(
        "\n\n"
        "\t\t       PLX PCIe NT Sample Application\n"
        "\t\t               July 2008\n\n"
        );


    /************************************
     *         Select Device
     ***********************************/
    DeviceSelected =
        SelectDevice_NT(
            &DeviceKey,
            &PlxChip,
            &bLinkSide
            );

    if (DeviceSelected == -1)
    {
        ConsoleEnd();
        exit(0);
    }

    status =
        PlxPci_DeviceOpen(
            &DeviceKey,
            &Device
            );

    if (status != ApiSuccess)
    {
        Cons_printf(
            "\n"
            "    ERROR: Unable to find or select a PLX device\n"
            );

        goto _ExitApp;
    }

    Cons_clear();

    // Display test version information
    Cons_printf(
        "\n"
        "                 PLX Non-Transparent Sample\n"
        "    =================================================\n"
        );

    Cons_printf(
        "\nSelected: %.4x %.4x [b:%02x s:%02x f:%02x]\n\n",
        DeviceKey.DeviceId, DeviceKey.VendorId,
        DeviceKey.bus, DeviceKey.slot, DeviceKey.function
        );


    /************************************
     *     Display the NT side
     ***********************************/
    Cons_printf(
        "Communicating from: %s side\n\n",
        (bLinkSide) ? "LINK" : "VIRTUAL"
        );


    /*************************************************************
     * Get PCI BAR and map
     ************************************************************/
    // Only PCI BAR 2 is currently supported
    BarNum = 2;

    Cons_printf("PCI BAR %d Properties : ", BarNum);

    status =
        PlxPci_PciBarProperties(
            &Device,
            BarNum,
            &BarProp
            );

    if (status != ApiSuccess)
    {
        Cons_printf("ERROR: Unable to get PCI BAR properties\n");
        goto _ExitApp;
    }
    Cons_printf("Ok\n");

    // Map the BAR to a virtual address
    Cons_printf("Map BAR to user space: ");
    status =
        PlxPci_PciBarMap(
            &Device,
            BarNum,
            &BarVa
            );

    if (status != ApiSuccess)
    {
        Cons_printf("ERROR: Unable to map PCI BAR\n");
        goto _ExitApp;
    }
    Cons_printf("Ok\n");

    Cons_printf(
        "  Properties:\n"
        "    Address  : 0x%08lx\n"
        "    Size     : %ld KB\n"
        "    Type     : %s\n"
        "    Prefetch?: %s\n"
        "    64-bit?  : %s\n"
        "    Virt Addr: 0x%p\n\n",
        (PLX_UINT_PTR)BarProp.Physical,
        (PLX_UINT_PTR)(BarProp.Size >> 10),
        (BarProp.bIoSpace) ? "I/O Port" : "Memory Space",
        (BarProp.bPrefetchable) ? "Yes" : "No",
        (BarProp.b64bit) ? "Yes" : "No",
        BarVa
        );


    /*************************************************************
     * Determine Requester ID & add LUT entry
     ************************************************************/
    Cons_printf("Probe to get Requester ID: ");

    if (PlxDetermineRequesterID(
            &Device,
            BarVa,
            &ReqId
            ) == FALSE)
    {
        Cons_printf("-- Skipping LUT entry update --\n\n");
    }
    else
    {
        Cons_printf("Ok\n");

        Cons_printf("Add Req ID entry in LUT  : ");

        if (PlxAddLutEntry(
                &Device,
                0,          // Use LUT entry 0
                ReqId,
                PlxChip,
                bLinkSide
                ) == FALSE)
        {
            goto _ExitApp;
        }
        Cons_printf("Ok\n\n");

        Cons_printf(
            "  Req ID = %04X\n"
            "  Device = b:%02X s:%02X f:%02X\n\n",
            ReqId,
            (ReqId >> 8) & 0xFF,
            (ReqId >> 3) & 0x1F,
            (ReqId >> 0) & 0x03
            );
    }


    /*************************************************************
     * Allocate & map PCI buffer
     ************************************************************/
    Cons_printf("Allocate PCI buffer: ");

    // Set desired size of buffer
    PhysBuffer.Size = NT_PCI_BUFFER_SIZE;

    status =
        PlxPci_PhysicalMemoryAllocate(
            &Device,
            &PhysBuffer,
            TRUE
            );

    if (status != ApiSuccess)
    {
        Cons_printf("ERROR: Unable to allocate a PCI buffer for data transfers\n");
        goto _ExitApp;
    }
    Cons_printf("Ok\n");

    Cons_printf("Map PCI buffer     : ");
    status =
        PlxPci_PhysicalMemoryMap(
            &Device,
            &PhysBuffer
            );

    if (status != ApiSuccess)
    {
        Cons_printf("ERROR: Unable to map system memory to user space\n");
        goto _ExitApp;
    }
    Cons_printf("Ok\n");

    Cons_printf(
        "  Properties:\n"
        "    PCI Addr : 0x%08lx\n"
        "    Size     : %ld bytes\n"
        "    Virt Addr: 0x%p\n\n",
        (PLX_UINT_PTR)PhysBuffer.PhysicalAddr,
        (PLX_UINT_PTR)PhysBuffer.Size,
        PLX_INT_TO_PTR(PhysBuffer.UserAddr)
        );

    // Setup a pointer to the buffer
    pSysMem = PLX_INT_TO_PTR(PhysBuffer.UserAddr);



    /*************************************************************
     * Post buffer properties for other side
     ************************************************************/
    Cons_printf(
        "Post buffer properties to %s side: ",
        (bLinkSide) ? "Link" : "Virtual"
        );

    // Set mailbox to write
    if (bLinkSide)
        value = 7;
    else
        value = 5;

    // Post buffer address
    PlxPci_PlxMailboxWrite(
        &Device,
        (S8)value,
        (U32)PhysBuffer.PhysicalAddr,
        PlxChip
        );

    Cons_printf("Ok\n");



    /*************************************************************
     * Establish connection with other side
     ************************************************************/
    if (WaitForConnection(
            &Device,
            PlxChip,
            bLinkSide
            ) != 0)
    {
        Cons_printf("ERROR: Unable to establish connection\n");
        goto _ExitApp;
    }



    /*************************************************************
     * Get buffer information from other side
     ************************************************************/
    Cons_printf("Get buffer address  : ");

    // Set mailbox to read
    if (bLinkSide)
        value = 5;
    else
        value = 7;

    // Get buffer address
    PhysAddr =
        PlxPci_PlxMailboxRead(
            &Device,
            (S8)value,
            NULL,
            PlxChip
            );

    Cons_printf("Ok (0x%08lx)\n", PhysAddr);



    /*************************************************************
     * Set up NT translation
     ************************************************************/
    Cons_printf("Setup NT translation: ");

    // Convert BAR size to range 
    size = ~((U32)BarProp.Size - 1);

    // Calculate BAR offset
    BarOffset = PhysAddr & ~size;

    // Calculate base address for translation
    PhysAddr &= size;

    // Prepare for NT translation setup
    DirAddr.DestinationAddr = PhysAddr;
    DirAddr.Size            = BarProp.Size;

    // Set BAR direct address translation
    status =
        PlxPci_SetupNtTranslation(
            &Device,
            BarNum,
            DirAddr,
            bLinkSide
            );

    if (status != ApiSuccess)
    {
        Cons_printf("ERROR: Unable to setup NT translation\n");
        goto _ExitApp;
    }
    Cons_printf("Ok\n");



    /*************************************************************
     * Main loop - send & receive key presses from other side
     ************************************************************/
    Cons_printf(
        "\n"
        "Type text to send to opposite NT port (ESC to halt)\n"
        "----------------------------------------------------------\n"
        "\n"
        );

    // Clear any pending characters
    *(VU32*)pSysMem = 0;

    do
    {
        // Check for key press
        if (Cons_kbhit())
        {
            // Get the character
            value = Cons_getch();

            // Write character to other side
            PlxBarMem_32(BarVa, BarOffset) = value;

            // Exit if ESC pressed
            if (value == 27)
            {
                Cons_printf("\n\n   -- User halt, closing connection -- \n");
                break;
            }
        }
            
        // Check if character passed from other side
        if (*(VU32*)pSysMem != 0)
        {
            // Get next character
            CharRead = (char)*(VU32*)pSysMem;

            // Clear character
            *(VU32*)pSysMem = 0;

            if ((isgraph(CharRead)) || (CharRead == ' '))
            {
                Cons_printf("%c", CharRead);
            }
            else if ((CharRead == '\r') || (CharRead == '\n'))
            {
                Cons_printf("\n");
            }
            else if (CharRead == '\b')
            {
                Cons_printf("\b \b");
            }
            else if (CharRead == 27)
            {
                // Other side terminated connection
                Cons_printf("\n\n    -- Other side terminated, closing connection -- \n");
                break;
            }
        }

        // Small delay
        Plx_sleep(10);
    }
    while (1);

    Cons_printf("\n\n");


_ExitApp:

    if (PhysBuffer.PhysicalAddr != 0)
    {
        Cons_printf("Release resources: ");

        // Free system memory
        PlxPci_PhysicalMemoryFree(
            &Device,
            &PhysBuffer
            );

        Cons_printf("Ok\n");
    }

    Cons_printf("\nPress any key to exit...\n");
    Cons_getch();

    ConsoleEnd();

    return 0;
}




/*********************************************************************
 *
 * Function   : SelectDevice_NT
 *
 * Description: Asks the user which PLX NT port device to select
 *
 * Returns    : Total devices found
 *              -1,  if user cancelled the selection
 *
 ********************************************************************/
S8
SelectDevice_NT(
    PLX_DEVICE_KEY *pKey,
    U32            *pPlxChip,
    BOOLEAN        *pbLinkSide
    )
{
    S32               i;
    S32               NumDevices;
    U8                Revision;
    U32               ChipType;
    U32               PlxChip[MAX_DEVICES_TO_LIST];
    BOOLEAN           bAddDevice;
    BOOLEAN           bLinkSide[MAX_DEVICES_TO_LIST];
    PLX_STATUS        status;
    PLX_PORT_PROP     PortProp;
    PLX_DEVICE_KEY    DevKey;
    PLX_DEVICE_KEY    DevKey_NT[MAX_DEVICES_TO_LIST];
    PLX_DRIVER_PROP   DriverProp;
    PLX_DEVICE_OBJECT Device;


    Cons_printf("\n");

    i          = 0;
    NumDevices = 0;

    do
    {
        // Setup for next device
        memset(&DevKey, PCI_FIELD_IGNORE, sizeof(PLX_DEVICE_KEY));

        // Check if device exists
        status =
            PlxPci_DeviceFind(
                &DevKey,
                (U8)i
                );

        if (status == ApiSuccess)
        {
            // Default to add device
            bAddDevice = TRUE;

            // Open device to get its properties
            PlxPci_DeviceOpen(
                &DevKey,
                &Device
                );

            // Get chip type
            PlxPci_ChipTypeGet(
                &Device,
                &ChipType,
                &Revision
                );

            // Verify supported chip type
            if (((ChipType & 0xFF00) != 0x8500) &&
                ((ChipType & 0xFF00) != 0x8600))
            {
                bAddDevice = FALSE;
            }

            // Verify device is NT port
            if (bAddDevice)
            {
                // Get port properties
                PlxPci_GetPortProperties(
                    &Device,
                    &PortProp
                    );

                if (PortProp.PortType != PLX_PORT_NON_TRANS)
                {
                    bAddDevice = FALSE;
                }
            }

            // Verify driver used in NT PnP and not Service driver
            if (bAddDevice)
            {
                PlxPci_DriverProperties(
                    &Device,
                    &DriverProp
                    );

                if (DriverProp.bIsServiceDriver)
                {
                    bAddDevice = FALSE;
                }
            }

            // Determine whether Virtual or Link-side port
            if (bAddDevice)
            {
                if (PlxDetermineNtPortSide(
                        &Device,
                        &bLinkSide[NumDevices]
                        ) == FALSE)
                {
                    bAddDevice = FALSE;
                }
            }

            // Close device
            PlxPci_DeviceClose(
                &Device
                );

            if (bAddDevice)
            {
                // Copy device key info
                DevKey_NT[NumDevices] = DevKey;
                PlxChip[NumDevices]   = ChipType;

                Cons_printf(
                    "\t\t  %2d. %04x Port %d [b:%02x s:%02x] (%s-side)\n",
                    (NumDevices + 1), ChipType, PortProp.PortNumber,
                    DevKey.bus, DevKey.slot,
                    (bLinkSide[NumDevices]) ? "Link" : "Virtual"
                    );

                // Increment to next device
                NumDevices++;
            }

            i++;
        }
    }
    while ((status == ApiSuccess) && (NumDevices < MAX_DEVICES_TO_LIST));

    if (NumDevices == 0)
        return 0;

    Cons_printf(
        "\t\t   0. Cancel\n\n"
        );

    do
    {
        Cons_printf("\t  Device Selection --> ");

        Cons_scanf("%d", &i);
    }
    while (i > NumDevices);

    if (i == 0)
        return -1;

    // Return selected device information
    *pKey       = DevKey_NT[i - 1];
    *pPlxChip   = PlxChip[i - 1];
    *pbLinkSide = bLinkSide[i - 1];

    return (S8)NumDevices;
}




/******************************************************************************
 *
 * Function   :  PlxDetermineNtPortSide
 *
 * Description:  Determines which side of the NT port is selected
 *
 *****************************************************************************/
BOOLEAN
PlxDetermineNtPortSide(
    PLX_DEVICE_OBJECT *pDevice,
    BOOLEAN           *pbLinkSide
    )
{
    U32 RegValue;
    U32 RegInternal;


    // Default to virtual side
    *pbLinkSide = FALSE;

    // Read BAR 0
    RegValue =
        PlxPci_PciRegisterReadFast(
            pDevice,
            0x10,       // BAR 0
            NULL
            );

    if (RegValue == (U32)-1)
        return FALSE;

    // If BAR 0 not enabled, then this is 8500 virtual-side
    if (RegValue == 0)
        return TRUE;

    // Read internal link-side BAR 0
    RegInternal =
        PlxPci_PlxMappedRegisterRead(
            pDevice,
            0x11000 + 0x10,    // NT Link-side BAR 0
            NULL
            );

    if ((RegInternal == 0) || (RegInternal == (U32)-1))
        return FALSE;

    // If internal BAR 0 matches PCI BAR 0, port is link side
    if (RegValue == RegInternal)
        *pbLinkSide = TRUE;

    return TRUE;
}




/******************************************************************************
 *
 * Function   :  PlxDetermineRequesterID
 *
 * Description:  Implements a procedure to determine the master ID for NT reads
 *
 *****************************************************************************/
BOOLEAN
PlxDetermineRequesterID(
    PLX_DEVICE_OBJECT *pDevice,
    U32               *VaBar2,
    U16               *pReqId
    )
{
    U32 RegValue;


    /*****************************************************
     * This function
     * 
     * 
     * 
     ****************************************************/

    // Default to Req ID of 0
    *pReqId = 0;

    // Get uncorrectable error status
    RegValue =
        PlxPci_PciRegisterReadFast(
            pDevice,
            0xfb8,       // AER Uncorrectable error status
            NULL
            );

    if (RegValue == -1)
    {
        Cons_printf("ERROR - Unable to read AER error status\n");
        return FALSE;
    }

    // Clear current error status
    if (RegValue != 0)
    {
        PlxPci_PciRegisterWriteFast(
            pDevice,
            0xfb8,       // AER Uncorrectable error status
            RegValue
            );
    }

    // Issue a dummy write to NT BAR 2
    PlxBarMem_32(VaBar2, 0) = 0;

    // Verify an uncorrectable error was flagged
    RegValue =
        PlxPci_PciRegisterReadFast(
            pDevice,
            0xfb8,       // AER Uncorrectable error status
            NULL
            );

    if (RegValue == 0)
    {
        Cons_printf("ERROR - Unable to generate error TLP\n");
        return FALSE;
    }

    // Get the error TLP DWord 1, which contains Req ID
    RegValue =
        PlxPci_PciRegisterReadFast(
            pDevice,
            0xfd4,       // AER TLP Header 1
            NULL
            );

    // Get Requester ID (DW1[31:16])
    *pReqId = (U16)(RegValue >> 16);

    return TRUE;
}




/******************************************************************************
 *
 * Function   :  PlxAddLutEntry
 *
 * Description:  Adds an LUT entry for the provided Requester ID
 *
 *****************************************************************************/
BOOLEAN
PlxAddLutEntry(
    PLX_DEVICE_OBJECT *pDevice,
    U16                index,
    U16                ReqId,
    U32                PlxChip,
    BOOLEAN            bLinkSide
    )
{
    U16 offset;
    U32 LutEntry;
    U32 LutValue;


    // Set the starting offset for the LUT
    switch (PlxChip)
    {
        case 0x8508:
        case 0x8512:
        case 0x8516:
        case 0x8517:
        case 0x8518:
        case 0x8524:
        case 0x8532:
            if (bLinkSide)
                offset = 0xDB4;
            else
                offset = 0xC5C + (index * sizeof(U32));
            break;

        default:
            if (bLinkSide)
                offset = 0xDB4;
            else
                offset = 0xD94 + (index * sizeof(U32));
            break;
    }

    // Prepare new LUT entry
    if (bLinkSide)
    {
        // Build LUT entry
        LutEntry =
            (1              << 0) |     // LUT entry enable
            (1              << 1) |     // Enable No_Snoop for entry
            ((ReqId & ~0x7) << 0);      // Requester ID (function # not allowed)

        // Get the current LUT entry
        LutValue =
            PlxPci_PlxRegisterRead(
                pDevice,
                offset,
                NULL
                );

        // Insert LUT entry in correct portion of 32-bit value
        if (index & (1 << 0))
        {
            LutValue &= 0x0000FFFF;
            LutValue |= (LutEntry << 16);
        }
        else
        {
            LutValue &= 0xFFFF0000;
            LutValue |= (LutEntry << 0);
        }
    }
    else
    {
        LutValue =
            (1     << 31) |     // LUT entry enable
            (1     << 30) |     // Enable No_Snoop for entry
            (ReqId <<  0);      // Requester ID
    }

    // Update the LUT entry
    PlxPci_PlxRegisterWrite(
        pDevice,
        offset,
        LutValue
        );

    return TRUE;
}




/******************************************************************************
 *
 * Function   :  WaitForConnection
 *
 * Description:  
 *
 *****************************************************************************/
S8
WaitForConnection(
    PLX_DEVICE_OBJECT *pDevice,
    U32                PlxChip,
    BOOLEAN            bLinkSide
    )
{
    char DispStat[] = "/|\\-";
    S16  LoopCount;
    U16  RegReady;
    U16  RegOffset;
    U32  RegValue;


    Cons_printf(
        "Wait for %s side (ESC to cancel): ",
        (bLinkSide) ? "Virtual" : "Link"
        );

    // Set register offset to use
    if ((PlxChip & 0xFF00) == 0x8600)
    {
        if (bLinkSide)
        {
            RegOffset = PLX_8600_NT_SCRATCH_REG_4;
            RegReady  = PLX_8600_NT_SCRATCH_REG_6;
        }
        else
        {
            RegOffset = PLX_8600_NT_SCRATCH_REG_6;
            RegReady  = PLX_8600_NT_SCRATCH_REG_4;
        }
    }
    else
    {
        if (bLinkSide)
        {
            RegOffset = PLX_8500_NT_SCRATCH_REG_4;
            RegReady  = PLX_8500_NT_SCRATCH_REG_6;
        }
        else
        {
            RegOffset = PLX_8500_NT_SCRATCH_REG_6;
            RegReady  = PLX_8500_NT_SCRATCH_REG_4;
        }
    }

    // Notify other side ready to connect
    PlxPci_PlxRegisterWrite(
        pDevice,
        RegReady,
        NT_MSG_SYSTEM_READY
        );

    // Set counter
    LoopCount = 180;

    // Wait for other side to respond
    do
    {
        Cons_printf(
            "%c\b",
            DispStat[(LoopCount % 4)]
            );

        LoopCount--;

        // Small delay
        Plx_sleep(500);

        // Check for a message
        RegValue =
            PlxPci_PlxRegisterRead(
                pDevice,
                RegOffset,
                NULL
                );

        // Check for ESC key to cancel
        if (Cons_kbhit())
        {
            if (Cons_getch() == 27)
            {
                LoopCount = -1;
            }
        }
    }
    while ((LoopCount > 0) && (RegValue != NT_MSG_SYSTEM_READY));

    // Clear message
    PlxPci_PlxRegisterWrite(
        pDevice,
        RegOffset,
        0
        );

    if (LoopCount == 0)
    {
        Cons_printf("ERROR - Timeout\n");
        return -1;
    }

    if (LoopCount == -1)
    {
        Cons_printf("User cancelled\n");
        return -1;
    }

    Cons_printf("Ok\n");
    Cons_printf("\n    -- Connection established! --\n\n");

    return 0;
}




/******************************************************************************
 *
 * Function   :  PlxPci_SetupNtTranslation
 *
 * Description:  
 *
 *****************************************************************************/
PLX_STATUS
PlxPci_SetupNtTranslation(
    PLX_DEVICE_OBJECT  *pDevice,
    U8                  BarIndex,
    PLX_DIRECT_ADDRESS  DirAddr,
    BOOLEAN             bLinkSide
    )
{
    U16 DestBaseLow;
    U16 DestBaseHi;
    U16 DestLimitLow;
    U16 DestLimitHi;
    U32 value;


    // Only BAR 2 currently supported
    if (BarIndex != 2)
        return ApiUnsupportedFunction;

    if (bLinkSide)
    {
        if (BarIndex == 2)
        {
            if (((DirAddr.DestinationAddr >> 32) != 0) ||
                 ((DirAddr.Size >> 32) != 0))
            {
                return ApiInvalidAddress;
            }
        }
    }

    if (BarIndex == 2)
    {
        DestBaseLow  = NT_MEMORY_BAR_2_TRAN_LOWER;
        DestBaseHi   = NT_MEMORY_BAR_2_TRAN_UPPER;
        DestLimitLow = NT_MEMORY_BAR_2_LIMIT_LOWER;
        DestLimitHi  = NT_MEMORY_BAR_2_LIMIT_UPPER;
    }
    else
    {
        DestBaseLow  = NT_MEMORY_BAR_4_TRAN_LOWER;
        DestBaseHi   = NT_MEMORY_BAR_4_TRAN_UPPER;
        DestLimitLow = NT_MEMORY_BAR_4_LIMIT_LOWER;
        DestLimitHi  = NT_MEMORY_BAR_4_LIMIT_UPPER;
    }

    PlxPci_PlxRegisterWrite(
        pDevice,
        DestBaseLow,
        (U32)DirAddr.DestinationAddr
        );

    PlxPci_PlxRegisterWrite(
        pDevice,
        DestBaseHi,
        (U32)(DirAddr.DestinationAddr >> 32)
        );

    // @@@ watch out for wrap arounds!!! think about this later - maybe give an error on wrap around
    value = (U32)DirAddr.DestinationAddr + (U32)DirAddr.Size;
    PlxPci_PlxRegisterWrite(
        pDevice,
        DestLimitLow,
        value
        );

    value = (U32)((DirAddr.DestinationAddr >> 32) + (DirAddr.Size >> 32));
    PlxPci_PlxRegisterWrite(
        pDevice,
        DestLimitHi,
        value
        );

    return ApiSuccess;
}




/******************************************************************************
 *
 * Function   :  PlxPci_PlxMailboxRead
 *
 * Description:  Function to read a PLX mailbox register
 *
 *****************************************************************************/
U32
PlxPci_PlxMailboxRead(
    PLX_DEVICE_OBJECT *pDevice,
    S8                 Mailbox,
    PLX_STATUS        *pStatus,
    U32                PlxChip
    )
{
    U16 offset;


    // Verify valid mailbox
    if ((Mailbox < 0) || (Mailbox > 7))
    {
        if (pStatus != NULL)
            *pStatus = ApiInvalidOffset;
        return 0;
    }

    // Set mailbox register base
    if ((PlxChip & 0xFF00) == 0x8600)
        offset = PLX_8600_NT_SCRATCH_REG_0;
    else
        offset = PLX_8500_NT_SCRATCH_REG_0;

    // Set status code
    if (pStatus != NULL)
        *pStatus = ApiSuccess;

    // Calculate desired mailbox offset
    offset = offset + (Mailbox * sizeof(U32));

    // Write buffer address to mailbox
    return PlxPci_PlxRegisterRead(
        pDevice,
        offset,
        pStatus
        );
}




/******************************************************************************
 *
 * Function   :  PlxPci_PlxMailboxRead
 *
 * Description:  Function to read a PLX mailbox register
 *
 *****************************************************************************/
PLX_STATUS
PlxPci_PlxMailboxWrite(
    PLX_DEVICE_OBJECT *pDevice,
    S8                 Mailbox,
    U32                value,
    U32                PlxChip
    )
{
    U16 offset;


    // Verify valid mailbox
    if ((Mailbox < 0) || (Mailbox > 7))
    {
        return ApiInvalidOffset;
    }

    // Set mailbox register base
    if ((PlxChip & 0xFF00) == 0x8600)
        offset = PLX_8600_NT_SCRATCH_REG_0;
    else
        offset = PLX_8500_NT_SCRATCH_REG_0;

    // Calculate desired mailbox offset
    offset = offset + (Mailbox * sizeof(U32));

    // Write buffer address to mailbox
    PlxPci_PlxRegisterWrite(
        pDevice,
        offset,
        value
        );

    return ApiSuccess;
}
