#ifndef _PCI_DEVICE_H
#define _PCI_DEVICE_H

/******************************************************************************
 *
 * File Name:
 *
 *      PciDevice.h
 *
 * Description:
 *
 *      Definitions for PCI functions
 *
 ******************************************************************************/


#include "PlxApi.h"

#if defined(PLX_DOS)
    #include "PciFunc.h"
#endif




/*************************************
 *          Definitions
 ************************************/
#if defined(PLX_DOS)
    #define PlxCm_MemRead_8(addr)           PHYS_MEM_READ_8 ((addr))
    #define PlxCm_MemRead_16(addr)          PHYS_MEM_READ_16((addr))
    #define PlxCm_MemRead_32(addr)          PHYS_MEM_READ_32((addr))

    #define PlxCm_MemWrite_8(addr , value)  PHYS_MEM_WRITE_8 ((addr), (value))
    #define PlxCm_MemWrite_16(addr, value)  PHYS_MEM_WRITE_16((addr), (value))
    #define PlxCm_MemWrite_32(addr, value)  PHYS_MEM_WRITE_32((addr), (value))
#else
    #define PlxCm_MemRead_8(addr)           (*(U8*)(addr))
    #define PlxCm_MemRead_16(addr)          (*(U16*)(addr))
    #define PlxCm_MemRead_32(addr)          (*(U32*)(addr))

    #define PlxCm_MemWrite_8(addr , value)  (*(U8*)(addr)  = (U8)(value))
    #define PlxCm_MemWrite_16(addr, value)  (*(U16*)(addr) = (U16)(value))
    #define PlxCm_MemWrite_32(addr, value)  (*(U32*)(addr) = (U32)(value))
#endif

#define MATCH_BASE_EXACT          ((U16)1 << 15)
#define MATCH_BASE_GENERIC        ((U16)1 << 12)
#define MATCH_BASE                MATCH_BASE_EXACT | MATCH_BASE_GENERIC

#define MATCH_SUBCLASS_EXACT      ((U16)1 << 11)
#define MATCH_SUBCLASS_GENERIC    ((U16)1 <<  8)
#define MATCH_SUBCLASS            MATCH_SUBCLASS_EXACT | MATCH_SUBCLASS_GENERIC

#define MATCH_INTERFACE_EXACT     ((U16)1 << 7)
#define MATCH_INTERFACE_GENERIC   ((U16)1 << 4)
#define MATCH_INTERFACE           MATCH_INTERFACE_EXACT | MATCH_INTERFACE_GENERIC

// Function number passed to driver in upper 3-bits of slot
#define PCI_DEVFN(slot, fn)       ((char)((((char)(fn)) << 5) | ((char)(slot) & 0x1f)))


typedef struct _DEVICE_NODE
{
    PLX_DEVICE_KEY       Key;
    U32                  PlxChipType;                   // PLX Chip type
    U8                   PlxRevision;                   // PLX Chip revision
    U8                   PciHeaderType;                 // Type of PCI Config registers
    U32                  PciClass;                      // PCI Device Class codes
    PLX_PORT_PROP        PortProp;                      // Port properties
    BOOLEAN              bSelected;                     // Flag to specify if device is selected
    PLX_UINT_PTR         Va_PciBar[6];                  // Virtual addresses of PCI BAR spaces
    struct _DEVICE_NODE *pNext;                         // Pointer to next node in device list
} DEVICE_NODE;




/*************************************
 *            Globals
 ************************************/
extern DEVICE_NODE *pDeviceList;




/*************************************
 *            Functions
 ************************************/
void
DeviceListCreate(
    void
    );

void
DeviceListDisplay(
    void
    );

void
DeviceListFree(
    void
    );

DEVICE_NODE *
DeviceNodeAdd(
    PLX_DEVICE_KEY *pKey
    );

BOOL
DeviceNodeExist(
    PLX_DEVICE_KEY *pKey
    );

void
DevicePropertiesFill(
    DEVICE_NODE *pNode
    );

DEVICE_NODE*
GetDevice(
    U8      index,
    BOOLEAN bPlxOnly
    );

VOID
Device_GetClassString(
    DEVICE_NODE *pNode,
    char        *pClassText
    );

BOOLEAN
Plx_EepromFileLoad(
    char    *pFileName,
    U16      EepSize,
    U8       EepPortSize,
    BOOLEAN  bCrc
    );

BOOLEAN
Plx_EepromFileSave(
    char    *pFileName,
    U32      PlxChip,
    U32      EepSize,
    U8       EepPortSize,
    BOOLEAN  bCrc
    );

BOOLEAN
Plx8000_EepromFileLoad(
    char *pFileName
    );

BOOLEAN
Plx8000_EepromFileSave(
    char *pFileName,
    U32   ByteCount
    );



#endif
